@@ -1,9 +1,8 @@
-from six import iteritems
 from graphql.language.visitor import Visitor
+from graphene import Union
 from graphene.utils.str_converters import to_snake_case
 from djraphql.django_models import get_field_by_name
 from graphql.execution.values import get_argument_values
-from graphql.language.ast import FragmentSpreadNode
 
 IGNORED_FIELDS = set(["__typename"])
 
@@ -17,11 +16,6 @@ class QueryFieldsVisitor(Visitor):
         self.graphql_type_stack = [info.parent_type]
         self.model_class_stack = []
 
-    def _get_terminal_gql_type(self, gql_type):
-        while hasattr(gql_type, "of_type"):
-            gql_type = gql_type.of_type
-        return gql_type
-
     # By returning the referenced fragment, we instruct GraphQL to continue
     # traversing the document contained within the fragment itself. This is
     # necessary to ensure queries that use fragments are correctly optimized.
@@ -65,6 +59,12 @@ class QueryFieldsVisitor(Visitor):
                 # If this ComputedField defined a depends_on list, we process it here.
                 entity_class = self.type_lookup[parent_gql_type.name]["entity_class"]
                 for computed_field in entity_class._get_computed_fields():
+
+                    # If we run into a Union type as a ComputedField, just skip this
+                    # because it's impossible to optimize the associated subtree.
+                    if is_union_type(computed_field.graphene_type):
+                        return False
+
                     for dependency in computed_field.depends_on:
                         field = try_get_related_field(model_class, dependency)
                         if field:
@@ -78,7 +78,7 @@ class QueryFieldsVisitor(Visitor):
             maybe_nonterminal_gql_type = parent_gql_type.fields.get(
                 node.name.value
             ).type
-            next_gql_type = self._get_terminal_gql_type(maybe_nonterminal_gql_type)
+            next_gql_type = get_terminal_gql_type(maybe_nonterminal_gql_type)
             self.graphql_type_stack.append(next_gql_type)
 
             if next_gql_type.name in self.type_lookup:
@@ -135,6 +135,22 @@ class QueryFieldsVisitor(Visitor):
                 self.goodbye_field(field)
 
 
+def get_terminal_gql_type(gql_type):
+    # It's common to wrap types in a lambda,
+    # so we unwrap the type here by calling the lambda
+    if callable(gql_type):
+        gql_type = gql_type()
+
+    while hasattr(gql_type, "of_type"):
+        gql_type = gql_type.of_type
+    return gql_type
+
+
+def is_union_type(gql_type):
+    terminal_type = get_terminal_gql_type(gql_type)
+    return isinstance(terminal_type, type) and issubclass(terminal_type, Union)
+
+
 def try_get_related_field(model_class, selection_name):
     names_to_try = [
         # First try just getting the field from the model via the selection's name
